@@ -37,11 +37,11 @@ static char * helplineStack[20];
 static char ** currentHelpline = NULL;
 
 static int cursorRow, cursorCol;
-static int needResize = 0;
 static int cursorOn = 1;
 static int trashScreen = 0;
+extern int needResize;
 
-static const char * defaultHelpLine =
+static const char * const defaultHelpLine =
 "  <Tab>/<Alt-Tab> between elements   |  <Space> selects   |  <F12> next screen"
 ;
 
@@ -53,21 +53,21 @@ const struct newtColors newtDefaultColorPalette = {
 	"red", "lightgray",			/* title fg, bg */
 	"lightgray", "red",			/* button fg, bg */
 	"red", "lightgray",			/* active button fg, bg */
-	"yellow", "blue",			/* checkbox fg, bg */
-	"blue", "brown",			/* active checkbox fg, bg */
-	"yellow", "blue",			/* entry box fg, bg */
+	"lightgray", "blue",			/* checkbox fg, bg */
+	"lightgray", "red",			/* active checkbox fg, bg */
+	"lightgray", "blue",			/* entry box fg, bg */
 	"blue", "lightgray",			/* label fg, bg */
 	"black", "lightgray",			/* listbox fg, bg */
-	"yellow", "blue",			/* active listbox fg, bg */
+	"lightgray", "blue",			/* active listbox fg, bg */
 	"black", "lightgray",			/* textbox fg, bg */
-	"lightgray", "black",			/* active textbox fg, bg */
+	"lightgray", "red",			/* active textbox fg, bg */
 	"white", "blue",			/* help line */
-	"yellow", "blue",			/* root text */
+	"lightgray", "blue",			/* root text */
 	"blue",					/* scale full */
 	"red",					/* scale empty */
 	"blue", "lightgray",			/* disabled entry fg, bg */
 	"black", "lightgray",			/* compact button fg, bg */
-	"yellow", "red",			/* active & sel listbox */
+	"lightgray", "red",			/* active & sel listbox */
 	"black", "brown"			/* selected listbox */
 };
 
@@ -131,6 +131,7 @@ static const struct keymap keymap[] = {
 	{ 0 },	/* LEAVE this one */
 };
 static void initKeymap();
+static void freeKeymap();
 
 static const char ident[] = // ident friendly
     "$Version: Newt windowing library v" VERSION " $"
@@ -184,12 +185,13 @@ int _newt_wstrlen(const char *str, int len) {
 void trim_string(char *title, int chrs)
 {
 	char *p = title;
-	int ln = chrs;
+	int ln;
 	int x = 0,y = 0;
 	wchar_t tmp;
 	mbstate_t ps;
 
 	memset(&ps, 0, sizeof(ps));
+	ln = strlen(title);
 
 	while (*p) {
 		x = mbrtowc(&tmp, p, ln, &ps);
@@ -198,12 +200,13 @@ void trim_string(char *title, int chrs)
 			return;
 		}
 		y = wcwidth(tmp);
-		if (y > ln) {
+		if (y > chrs) {
 			*p = '\0';
 			return;
 		} else {
 			p += x;
-			ln -= y;
+			ln -= x;
+			chrs -= y;
 		}
 	}	
 }
@@ -212,7 +215,7 @@ static int getkey() {
     int c;
 
     while ((c = SLang_getkey()) == '\xC') { /* if Ctrl-L redraw whole screen */
-        SLsmg_touch_lines (0, SLtt_Screen_Rows - 1);
+        SLsmg_touch_lines(0, SLtt_Screen_Rows);
         SLsmg_refresh();
     }
     return c;
@@ -262,10 +265,11 @@ void newtCls(void) {
  * @param redraw - boolean - should we redraw the screen?
  */
 void newtResizeScreen(int redraw) {
+    /* we can't redraw from scratch, just redisplay SLang screen */
     SLtt_get_screen_size();
-    SLsmg_reinit_smg();
+    /* SLsmg_reinit_smg(); */
     if (redraw) {
-        SLsmg_touch_lines (0, SLtt_Screen_Rows - 1);
+        SLsmg_touch_lines(0, SLtt_Screen_Rows);
         newtRefresh();
     }
 }
@@ -283,20 +287,20 @@ int newtInit(void) {
         if ((lang = getenv("LC_CTYPE")) == NULL)
             if ((lang = getenv("LANG")) == NULL)
                 lang = "";
+    /* slang doesn't support multibyte encodings except UTF-8,
+       avoid character corruption by redrawing the screen */
     if (strstr (lang, ".euc") != NULL)
 	trashScreen = 1;
 
     (void) strlen(ident);
 
+    SLutf8_enable(-1);
     SLtt_get_terminfo();
     SLtt_get_screen_size();
 
     MonoValue = getenv(MonoEnv);
-    if ( MonoValue == NULL ) {
-	SLtt_Use_Ansi_Colors = 1;
-    } else {
+    if ( MonoValue != NULL )
 	SLtt_Use_Ansi_Colors = 0;
-    }
 
     if ((ret = SLsmg_init_smg()) < 0)
 	return ret;
@@ -307,10 +311,6 @@ int newtInit(void) {
     newtCursorOff();
     initKeymap();
 
-    /*memset(&sa, 0, sizeof(sa));
-    sa.sa_handler = handleSigwinch;
-    sigaction(SIGWINCH, &sa, NULL);*/
-
     SLsignal_intr(SIGWINCH, handleSigwinch);
     SLang_getkey_intr_hook = getkeyInterruptHook;
 
@@ -322,6 +322,22 @@ int newtInit(void) {
  * @returns int , 0. (no errors reported)
  */
 int newtFinished(void) {
+    if (currentWindow) {
+	for (; currentWindow >= windowStack; currentWindow--) {
+	    free(currentWindow->buffer);
+	    free(currentWindow->title);
+	}
+	currentWindow = NULL;
+    }
+
+    if (currentHelpline) {
+	for (; currentHelpline >= helplineStack; currentHelpline--)
+	    free(*currentHelpline);
+	currentHelpline = NULL;
+    }
+
+    freeKeymap();
+
     SLsmg_gotorc(SLtt_Screen_Rows - 1, 0);
     newtCursorOn();
     SLsmg_refresh();
@@ -399,22 +415,27 @@ void newtSetColors(struct newtColors colors) {
 		   colors.selListboxBg);
 }
 
+void newtSetColor(int colorset, char *fg, char *bg) {
+    if (colorset < NEWT_COLORSET_ROOT || colorset > NEWT_COLORSET_SELLISTBOX ||
+	    !SLtt_Use_Ansi_Colors)
+	return;
+
+    SLtt_set_color(colorset, "", fg, bg);
+}
+
 /* Keymap handling - rewritten by Henning Makholm <henning@makholm.net>,
  * November 2003.
  */
 
 struct kmap_trie_entry {
+    char alloced; /* alloced/not first element in array */
     char c ;   /* character got from terminal */
     int code;  /* newt key, or 0 if c does not make a complete sequence */
     struct kmap_trie_entry *contseq; /* sub-trie for character following c */
     struct kmap_trie_entry *next;    /* try this if char received != c */
 };
-/* Here are some static entries that will help in handling esc O foo and
-   esc [ foo as variants of each other: */
-static struct kmap_trie_entry
-    kmap_trie_escO     = { 'O', 0, 0, 0 },
-    kmap_trie_escBrack = { '[', 0, 0, &kmap_trie_escO },
-    kmap_trie_root     = { '\033', 0, &kmap_trie_escBrack, 0 };
+
+static struct kmap_trie_entry *kmap_trie_root = NULL;
 static int keyreader_buf_len = 10 ;
 static unsigned char default_keyreader_buf[10];
 static unsigned char *keyreader_buf = default_keyreader_buf;
@@ -452,7 +473,7 @@ static void dumpkeys_recursive(struct kmap_trie_entry *curr, int i, FILE *f) {
 static void dump_keymap(void) {
     FILE *f = fopen("newt.keydump","wt");
     if (f) {
-        dumpkeys_recursive(&kmap_trie_root,0,f);
+        dumpkeys_recursive(kmap_trie_root, 0, f);
         fclose(f);
     }
 }
@@ -460,7 +481,7 @@ static void dump_keymap(void) {
 
 /* newtBindKey may overwrite a binding that is there already */
 static void newtBindKey(char *keyseq, int meaning) {
-    struct kmap_trie_entry *root = &kmap_trie_root ;
+    struct kmap_trie_entry *root = kmap_trie_root ;
     struct kmap_trie_entry **curptr = &root ;
 
     /* Try to make sure the common matching buffer is long enough. */
@@ -484,6 +505,7 @@ static void newtBindKey(char *keyseq, int meaning) {
             struct kmap_trie_entry* fresh
                 =  calloc(strlen(keyseq),sizeof(struct kmap_trie_entry));
             if (fresh == 0) return; /* despair! */
+	    fresh->alloced = 1;
             *curptr = fresh;
             while (keyseq[1]) {
                 fresh->contseq = fresh+1;
@@ -524,6 +546,7 @@ static void kmap_trie_fallback(struct kmap_trie_entry *to,
             *fromcopy = malloc(sizeof(struct kmap_trie_entry));
             if (*fromcopy) {
                 **fromcopy = *to ;
+		(*fromcopy)->alloced = 1;
                 (*fromcopy)->next = 0 ;
             }
         }
@@ -534,7 +557,7 @@ int newtGetKey(void) {
     int key;
     unsigned char *chptr = keyreader_buf, *lastmatch;
     int lastcode;
-    struct kmap_trie_entry *curr = &kmap_trie_root;
+    struct kmap_trie_entry *curr = kmap_trie_root;
 
     do {
 	key = getkey();
@@ -610,14 +633,14 @@ void newtClearKeyBuffer(void) {
 
 /**
  * Open a new window.
- * @param left. unsigned int Size; _not_ including border
- * @param top: unsigned int size, _not_ including border
+ * @param left. int Size; _not_ including border
+ * @param top: int size, _not_ including border
  * @param width unsigned int
  * @param height unsigned int
  * @param title - title string
- * @return zero on success (currently no errors reported)
+ * @return zero on success
  */
-int newtOpenWindow(unsigned int left, unsigned int top, 
+int newtOpenWindow(int left, int top, 
                    unsigned int width, unsigned int height,
 			  const char * title) {
     int j, row, col;
@@ -626,6 +649,10 @@ int newtOpenWindow(unsigned int left, unsigned int top,
 
     newtFlushInput();
 
+    if (currentWindow && currentWindow - windowStack + 1
+	    >= sizeof (windowStack) / sizeof (struct Window))
+	return 1;
+
     if (!currentWindow) {
 	currentWindow = windowStack;
     } else {
@@ -638,10 +665,10 @@ int newtOpenWindow(unsigned int left, unsigned int top,
     currentWindow->height = height;
     currentWindow->title = title ? strdup(title) : NULL;
 
-    currentWindow->buffer = malloc(sizeof(SLsmg_Char_Type) * (width + 3) * (height + 3));
+    currentWindow->buffer = malloc(sizeof(SLsmg_Char_Type) * (width + 5) * (height + 3));
 
     row = top - 1;
-    col = left - 1;
+    col = left - 2;
     /* clip to the current screen bounds - msw */
     if (row < 0)
 	row = 0;
@@ -655,8 +682,8 @@ int newtOpenWindow(unsigned int left, unsigned int top,
     for (j = 0; j < height + 3; j++, row++) {
 	SLsmg_gotorc(row, col);
 	SLsmg_read_raw(currentWindow->buffer + n,
-		       currentWindow->width + 3);
-	n += currentWindow->width + 3;
+		       currentWindow->width + 5);
+	n += currentWindow->width + 5;
     }
 
     newtTrashScreen();
@@ -704,35 +731,41 @@ int newtOpenWindow(unsigned int left, unsigned int top,
  * @param width - width in char cells
  * @param height - no. of char cells.
  * @param title - fixed title
- * @returns 0. No errors reported
+ * @returns zero on success
  */
 int newtCenteredWindow(unsigned int width,unsigned int height, 
                        const char * title) {
-    unsigned int top, left;
+    int top, left;
 
-    top = (SLtt_Screen_Rows - height) / 2;
+    top = (int)(SLtt_Screen_Rows - height) / 2;
 
     /* I don't know why, but this seems to look better */
     if ((SLtt_Screen_Rows % 2) && (top % 2)) top--;
 
-    left = (SLtt_Screen_Cols - width) / 2;
+    left = (int)(SLtt_Screen_Cols - width) / 2;
 
-    newtOpenWindow(left, top, width, height, title);
-
-    return 0;
+    return newtOpenWindow(left, top, width, height, title);
 }
 
 /**
  * @brief Remove the top window
  */
 void newtPopWindow(void) {
+    newtPopWindowNoRefresh();
+    newtRefresh();
+}
+
+void newtPopWindowNoRefresh(void) {
     int j, row, col;
     int n = 0;
 
+    if (currentWindow == NULL)
+	return;
+
     row = col = 0;
 
     row = currentWindow->top - 1;
-    col = currentWindow->left - 1;
+    col = currentWindow->left - 2;
     if (row < 0)
 	row = 0;
     if (col < 0)
@@ -740,8 +773,8 @@ void newtPopWindow(void) {
     for (j = 0; j < currentWindow->height + 3; j++, row++) {
 	SLsmg_gotorc(row, col);
 	SLsmg_write_raw(currentWindow->buffer + n,
-			currentWindow->width + 3);
-	n += currentWindow->width + 3;
+			currentWindow->width + 5);
+	n += currentWindow->width + 5;
     }
 
     free(currentWindow->buffer);
@@ -755,8 +788,6 @@ void newtPopWindow(void) {
     SLsmg_set_char_set(0);
 
     newtTrashScreen();
-
-    newtRefresh();
 }
 
 void newtGetWindowPos(int * x, int * y) {
@@ -768,8 +799,13 @@ void newtGetWindowPos(int * x, int * y) {
 }
 
 void newtGetrc(int * row, int * col) {
-   *row = cursorRow;
-   *col = cursorCol;
+    *row = cursorRow;
+    *col = cursorCol;
+
+    if (currentWindow) {
+	*row -= currentWindow->top;
+	*col -= currentWindow->left;
+    }
 }
 
 void newtGotorc(int newRow, int newCol) {
@@ -809,6 +845,22 @@ void newtClearBox(int left, int top, int width, int height) {
 
 static void initKeymap(void) {
     const struct keymap * curr;
+    struct kmap_trie_entry *kmap_trie_escBrack, *kmap_trie_escO;
+
+    /* Here are some entries that will help in handling esc O foo and
+       esc [ foo as variants of each other. */
+    kmap_trie_root = calloc(3, sizeof (struct kmap_trie_entry));
+    kmap_trie_escBrack = kmap_trie_root + 1;
+    kmap_trie_escO = kmap_trie_root + 2;
+
+    kmap_trie_root->alloced = 1;
+    kmap_trie_root->c = '\033';
+    kmap_trie_root->contseq = kmap_trie_escBrack;
+
+    kmap_trie_escBrack->c = '[';
+    kmap_trie_escBrack->next = kmap_trie_escO;
+
+    kmap_trie_escO->c = 'O';
 
     /* First bind built-in default bindings. They may be shadowed by
        the termcap entries that get bound later. */
@@ -836,8 +888,33 @@ static void initKeymap(void) {
        keypad modes. Or perhaps they were, but tried to make their
        description work with a program that puts the keyboard in the
        wrong emulation mode. In short, one needs this: */
-    kmap_trie_fallback(kmap_trie_escO.contseq, &kmap_trie_escBrack.contseq);
-    kmap_trie_fallback(kmap_trie_escBrack.contseq, &kmap_trie_escO.contseq);
+    kmap_trie_fallback(kmap_trie_escO->contseq, &kmap_trie_escBrack->contseq);
+    kmap_trie_fallback(kmap_trie_escBrack->contseq, &kmap_trie_escO->contseq);
+}
+
+static void free_keys(struct kmap_trie_entry *kmap, struct kmap_trie_entry *parent, int prepare) {
+    if (kmap == NULL)
+	return;
+
+    free_keys(kmap->contseq, kmap, prepare);
+    free_keys(kmap->next, kmap, prepare);
+
+    if (!kmap->alloced && kmap - parent == 1)
+	    return;
+
+    /* find first element in array */
+    while (!kmap->alloced)
+	kmap--;
+
+    kmap->alloced += prepare ? 1 : -1;
+    if (!prepare && kmap->alloced == 1)
+	free(kmap);
+}
+
+static void freeKeymap() {
+    free_keys(kmap_trie_root, NULL, 1);
+    free_keys(kmap_trie_root, NULL, 0);
+    kmap_trie_root = NULL;
 }
 
 /**
@@ -845,15 +922,7 @@ static void initKeymap(void) {
  * @param int - number of usecs to wait for.
  */
 void newtDelay(unsigned int usecs) {
-    fd_set set;
-    struct timeval tv;
-
-    FD_ZERO(&set);
-
-    tv.tv_sec = usecs / 1000000;
-    tv.tv_usec = usecs % 1000000;
-
-    select(0, &set, &set, &set, &tv);
+    usleep(usecs);
 }
 
 struct eventResult newtDefaultEventHandler(newtComponent c,
@@ -892,6 +961,10 @@ void newtRedrawHelpLine(void) {
 }
 
 void newtPushHelpLine(const char * text) {
+    if (currentHelpline && currentHelpline - helplineStack + 1
+	    >= sizeof (helplineStack) / sizeof (char *))
+	return;
+
     if (!text)
 	text = defaultHelpLine;
 
@@ -979,6 +1052,6 @@ void newtCursorOn(void) {
 
 void newtTrashScreen(void) {
     if (trashScreen)
-	SLsmg_touch_lines (0, SLtt_Screen_Rows - 1);
+	SLsmg_touch_lines(0, SLtt_Screen_Rows);
 }
      
