@@ -53,9 +53,11 @@ static PyObject * messageWindow(PyObject * s, PyObject * args);
 static PyObject * openWindow(PyObject * s, PyObject * args);
 static PyObject * popHelpLine(PyObject * s, PyObject * args);
 static PyObject * popWindow(PyObject * s, PyObject * args);
+static PyObject * popWindowNoRefresh(PyObject * s, PyObject * args);
 static PyObject * pushHelpLine(PyObject * s, PyObject * args);
 static snackWidget * radioButtonWidget(PyObject * s, PyObject * args);
 static PyObject * refreshScreen(PyObject * s, PyObject * args);
+static PyObject * setColor(PyObject * s, PyObject * args);
 static PyObject * scaleWidget(PyObject * s, PyObject * args);
 static PyObject * scaleSet(snackWidget * s, PyObject * args);
 static PyObject * screenSize(PyObject * s, PyObject * args);
@@ -87,10 +89,12 @@ static PyMethodDef snackModuleMethods[] = {
     { "openwindow", openWindow, METH_VARARGS, NULL },
     { "pophelpline", popHelpLine, METH_VARARGS, NULL },
     { "popwindow", popWindow, METH_VARARGS, NULL },
+    { "popwindownorefresh", popWindowNoRefresh, METH_VARARGS, NULL },
     { "pushhelpline", pushHelpLine, METH_VARARGS, NULL },
     { "radiobutton", (PyCFunction) radioButtonWidget, METH_VARARGS, NULL },
     { "reflow", (PyCFunction) reflowText, METH_VARARGS, NULL },
     { "refresh", refreshScreen, METH_VARARGS, NULL },
+    { "setcolor", setColor, METH_VARARGS, NULL },
     { "resume", doResume, METH_VARARGS, NULL },
     { "scale", scaleWidget, METH_VARARGS, NULL },
     { "size", screenSize, METH_VARARGS, NULL },
@@ -196,9 +200,11 @@ static PyObject * widgetListboxAdd(snackWidget * s, PyObject * args);
 static PyObject * widgetListboxIns(snackWidget * s, PyObject * args);
 static PyObject * widgetListboxDel(snackWidget * s, PyObject * args);
 static PyObject * widgetListboxGet(snackWidget * s, PyObject * args);
+static PyObject * widgetListboxGetSel(snackWidget * s, PyObject * args);
 static PyObject * widgetListboxSet(snackWidget * s, PyObject * args);
 static PyObject * widgetListboxClear(snackWidget * s, PyObject * args);
 static PyObject * widgetTextboxText(snackWidget * s, PyObject * args);
+static PyObject * widgetTextboxHeight(snackWidget * s, PyObject * args);
 static PyObject * widgetCheckboxTreeAddItem(snackWidget * s, PyObject * args);
 static PyObject * widgetCheckboxTreeGetSel(snackWidget * s, PyObject * args);
 static PyObject * widgetCheckboxTreeGetCur(snackWidget * s, PyObject * args);
@@ -215,10 +221,12 @@ static PyMethodDef widgetMethods[] = {
     { "setCallback", (PyCFunction) widgetAddCallback, METH_VARARGS, NULL },
     { "labelText", (PyCFunction) widgetLabelText, METH_VARARGS, NULL },
     { "textboxText", (PyCFunction) widgetTextboxText, METH_VARARGS, NULL },
+    { "textboxHeight", (PyCFunction) widgetTextboxHeight, METH_VARARGS, NULL },
     { "entrySetValue", (PyCFunction) widgetEntrySetValue, METH_VARARGS, NULL },
     { "listboxAddItem", (PyCFunction) widgetListboxAdd, METH_VARARGS, NULL },
     { "listboxInsertItem", (PyCFunction) widgetListboxIns, METH_VARARGS, NULL },
     { "listboxGetCurrent", (PyCFunction) widgetListboxGet, METH_VARARGS, NULL },
+    { "listboxGetSelection", (PyCFunction) widgetListboxGetSel, METH_VARARGS, NULL },
     { "listboxSetCurrent", (PyCFunction) widgetListboxSet, METH_VARARGS, NULL },
     { "listboxSetWidth", (PyCFunction) widgetListboxSetW, METH_VARARGS, NULL },
     { "listboxDeleteItem", (PyCFunction) widgetListboxDel, METH_VARARGS, NULL },
@@ -299,6 +307,18 @@ static PyObject * refreshScreen(PyObject * s, PyObject * args) {
     return Py_None;
 }
 
+static PyObject * setColor(PyObject * s, PyObject * args) {
+    char * fg, * bg;
+    int colorset;
+    if (!PyArg_ParseTuple(args, "iss", &colorset, &fg, &bg))
+	return NULL;
+
+    newtSetColor(colorset, fg, bg);
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
 static PyObject * scaleWidget(PyObject * s, PyObject * args) {
     snackWidget * widget;
     int width, fullAmount;
@@ -530,6 +550,12 @@ static PyObject * popWindow(PyObject * s, PyObject * args) {
     return Py_None;
 }
 
+static PyObject * popWindowNoRefresh(PyObject * s, PyObject * args) {
+    newtPopWindowNoRefresh();
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
 static PyObject * messageWindow(PyObject * s, PyObject * args) {
     char * title, * text;
     char * okbutton = "Ok";
@@ -629,19 +655,33 @@ static PyObject * widgetTextboxText(snackWidget * s, PyObject * args) {
     return Py_None;
 }
 
+static PyObject * widgetTextboxHeight(snackWidget * s, PyObject * args) {
+    int height;
+
+    if (!PyArg_ParseTuple(args, "i", &height)) return NULL;
+
+    newtTextboxSetHeight(s->co, height);
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
 static snackWidget * listboxWidget(PyObject * s, PyObject * args) {
     snackWidget * widget;
     int height;
-    int doScroll = 0, returnExit = 0, showCursor = 0 ;
+    int doScroll = 0, returnExit = 0, showCursor = 0, multiple = 0, border = 0;
 
-    if (!PyArg_ParseTuple(args, "i|iii", &height, &doScroll, &returnExit, &showCursor))
+    if (!PyArg_ParseTuple(args, "i|iiiii", &height, &doScroll, &returnExit, 
+					   &showCursor, &multiple, &border))
 	return NULL;
 
     widget = snackWidgetNew ();
     widget->co = newtListbox(-1, -1, height,
 			     (doScroll ? NEWT_FLAG_SCROLL : 0) |
 			     (returnExit ? NEWT_FLAG_RETURNEXIT : 0) |
-			     (showCursor ? NEWT_FLAG_SHOWCURSOR : 0)
+			     (showCursor ? NEWT_FLAG_SHOWCURSOR : 0) |
+			     (multiple ? NEWT_FLAG_MULTIPLE : 0) |
+			     (border ? NEWT_FLAG_BORDER : 0)
 			     );
     widget->anint = 1;
     
@@ -924,8 +964,8 @@ static void widgetDestructor(PyObject * o) {
     
     Py_XDECREF (s->scs.cb);
     Py_XDECREF (s->scs.data);
-
-    PyMem_DEL(o);
+    
+    PyObject_Free(o);
 }
 
 static PyObject * widgetAddCallback(snackWidget * s, PyObject * args) {
@@ -946,11 +986,12 @@ static PyObject * widgetAddCallback(snackWidget * s, PyObject * args) {
 
 static PyObject * widgetEntrySetValue(snackWidget * s, PyObject * args) {
     char * val;
+    int cursorAtEnd = 1;
 
-    if (!PyArg_ParseTuple(args, "s", &val))
+    if (!PyArg_ParseTuple(args, "s|i", &val, &cursorAtEnd))
 	return NULL;
 
-    newtEntrySet(s->co, val, 1);
+    newtEntrySet(s->co, val, cursorAtEnd);
 
     Py_INCREF(Py_None);
     return Py_None;
@@ -1010,6 +1051,32 @@ static PyObject * widgetListboxGet(snackWidget * s, PyObject * args) {
     return PyInt_FromLong((long) newtListboxGetCurrent(s->co));
 }
 
+static PyObject * widgetListboxGetSel(snackWidget * s, PyObject * args) {
+    void ** selection;
+    int numselected;
+    int i;
+    PyObject * sel;
+
+    if (!PyArg_ParseTuple(args, ""))
+        return NULL;
+
+    selection = (void **) newtListboxGetSelection(s->co, &numselected);
+
+    sel = PyList_New(0);
+
+    if (!selection) {
+        return sel;
+    }
+
+    sel = PyList_New(0);
+    for (i = 0; i < numselected; i++) {
+        PyList_Append(sel, PyInt_FromLong((long) selection[i]));
+    }
+    free(selection);
+
+    return sel;
+}
+
 static PyObject * widgetListboxSet(snackWidget * s, PyObject * args) {
     int index;
     
@@ -1220,9 +1287,11 @@ static PyObject * pywstrlen(PyObject * s, PyObject * args)
 void init_snack(void) {
     PyObject * d, * m;
 
+#ifdef ENABLE_NLS
     setlocale (LC_ALL, "");
     bindtextdomain (PACKAGE, LOCALEDIR);
     textdomain (PACKAGE);
+#endif
 
     m = Py_InitModule("_snack", snackModuleMethods);
     d = PyModule_GetDict(m);
@@ -1262,4 +1331,28 @@ void init_snack(void) {
     PyDict_SetItemString(d, "FLAGS_SET", PyInt_FromLong(NEWT_FLAGS_SET));
     PyDict_SetItemString(d, "FLAGS_RESET", PyInt_FromLong(NEWT_FLAGS_RESET));
     PyDict_SetItemString(d, "FLAGS_TOGGLE", PyInt_FromLong(NEWT_FLAGS_TOGGLE));
+
+    PyDict_SetItemString(d, "COLORSET_ROOT", PyInt_FromLong(NEWT_COLORSET_ROOT));
+    PyDict_SetItemString(d, "COLORSET_BORDER", PyInt_FromLong(NEWT_COLORSET_BORDER));
+    PyDict_SetItemString(d, "COLORSET_WINDOW", PyInt_FromLong(NEWT_COLORSET_WINDOW));
+    PyDict_SetItemString(d, "COLORSET_SHADOW", PyInt_FromLong(NEWT_COLORSET_SHADOW));
+    PyDict_SetItemString(d, "COLORSET_TITLE", PyInt_FromLong(NEWT_COLORSET_TITLE));
+    PyDict_SetItemString(d, "COLORSET_BUTTON", PyInt_FromLong(NEWT_COLORSET_BUTTON));
+    PyDict_SetItemString(d, "COLORSET_ACTBUTTON", PyInt_FromLong(NEWT_COLORSET_ACTBUTTON));
+    PyDict_SetItemString(d, "COLORSET_CHECKBOX", PyInt_FromLong(NEWT_COLORSET_CHECKBOX));
+    PyDict_SetItemString(d, "COLORSET_ACTCHECKBOX", PyInt_FromLong(NEWT_COLORSET_ACTCHECKBOX));
+    PyDict_SetItemString(d, "COLORSET_ENTRY", PyInt_FromLong(NEWT_COLORSET_ENTRY));
+    PyDict_SetItemString(d, "COLORSET_LABEL", PyInt_FromLong(NEWT_COLORSET_LABEL));
+    PyDict_SetItemString(d, "COLORSET_LISTBOX", PyInt_FromLong(NEWT_COLORSET_LISTBOX));
+    PyDict_SetItemString(d, "COLORSET_ACTLISTBOX", PyInt_FromLong(NEWT_COLORSET_ACTLISTBOX));
+    PyDict_SetItemString(d, "COLORSET_TEXTBOX", PyInt_FromLong(NEWT_COLORSET_TEXTBOX));
+    PyDict_SetItemString(d, "COLORSET_ACTTEXTBOX", PyInt_FromLong(NEWT_COLORSET_ACTTEXTBOX));
+    PyDict_SetItemString(d, "COLORSET_HELPLINE", PyInt_FromLong(NEWT_COLORSET_HELPLINE));
+    PyDict_SetItemString(d, "COLORSET_ROOTTEXT", PyInt_FromLong(NEWT_COLORSET_ROOTTEXT));
+    PyDict_SetItemString(d, "COLORSET_EMPTYSCALE", PyInt_FromLong(NEWT_COLORSET_EMPTYSCALE));
+    PyDict_SetItemString(d, "COLORSET_FULLSCALE", PyInt_FromLong(NEWT_COLORSET_FULLSCALE));
+    PyDict_SetItemString(d, "COLORSET_DISENTRY", PyInt_FromLong(NEWT_COLORSET_DISENTRY));
+    PyDict_SetItemString(d, "COLORSET_COMPACTBUTTON", PyInt_FromLong(NEWT_COLORSET_COMPACTBUTTON));
+    PyDict_SetItemString(d, "COLORSET_ACTSELLISTBOX", PyInt_FromLong(NEWT_COLORSET_ACTSELLISTBOX));
+    PyDict_SetItemString(d, "COLORSET_SELLISTBOX", PyInt_FromLong(NEWT_COLORSET_SELLISTBOX));
 }
