@@ -15,9 +15,10 @@ struct textbox {
     char *blankline;
     int linesAlloced;
     int doWrap;
-    newtComponent sb;
+    newtComponent sb_act, sb;
     int topLine;
     int textWidth;
+    int isActive;
 };
 
 static char * expandTabs(const char * text);
@@ -43,8 +44,10 @@ static void textboxMapped(newtComponent co, int isMapped) {
     struct textbox * tb = co->data;
 
     co->isMapped = isMapped;
-    if (tb->sb)
+    if (tb->sb) {
 	tb->sb->ops->mapped(tb->sb, isMapped);
+	tb->sb_act->ops->mapped(tb->sb_act, isMapped);
+    }
 }
 
 static void textboxPlace(newtComponent co, int newLeft, int newTop) {
@@ -53,8 +56,10 @@ static void textboxPlace(newtComponent co, int newLeft, int newTop) {
     co->top = newTop;
     co->left = newLeft;
 
-    if (tb->sb)
+    if (tb->sb) {
 	tb->sb->ops->place(tb->sb, co->left + co->width - 1, co->top);
+	tb->sb_act->ops->place(tb->sb_act, co->left + co->width - 1, co->top);
+    }
 }
 
 void newtTextboxSetHeight(newtComponent co, int height) {
@@ -95,11 +100,13 @@ newtComponent newtTextbox(int left, int top, int width, int height, int flags) {
 
     co->ops = &textboxOps;
 
+    co->isMapped = 0;
     co->height = height;
     co->top = top;
     co->left = left;
     co->takesFocus = 0;
     co->width = width;
+    co->destroyCallback = NULL;
 
     tb->doWrap = flags & NEWT_FLAG_WRAP;
     tb->numLines = 0;
@@ -107,16 +114,20 @@ newtComponent newtTextbox(int left, int top, int width, int height, int flags) {
     tb->lines = NULL;
     tb->topLine = 0;
     tb->textWidth = width;
+    tb->isActive = 0;
     tb->blankline = malloc(width+1);
     memset(tb->blankline,' ',width);
     tb->blankline[width] = '\0';
 
     if (flags & NEWT_FLAG_SCROLL) {
 	co->width += 2;
+	tb->sb_act = newtVerticalScrollbar(co->left + co->width - 1, co->top, 
+			   co->height, COLORSET_ACTTEXTBOX, COLORSET_TEXTBOX);
 	tb->sb = newtVerticalScrollbar(co->left + co->width - 1, co->top, 
 			   co->height, COLORSET_TEXTBOX, COLORSET_TEXTBOX);
+	co->takesFocus = 1;
     } else {
-	tb->sb = NULL;
+	tb->sb_act = tb->sb = NULL;
     }
 
     return co;
@@ -290,6 +301,10 @@ void newtTextboxSetText(newtComponent co, const char * text) {
     int badness, height;
 
     if (tb->lines) {
+	int i;
+
+	for (i = 0; i < tb->numLines; i++) 
+	    free(tb->lines[i]);
 	free(tb->lines);
 	tb->linesAlloced = tb->numLines = 0;
     }
@@ -320,6 +335,8 @@ void newtTextboxSetText(newtComponent co, const char * text) {
 
     free(expanded);
     
+    textboxDraw(co);
+
     newtTrashScreen();    
 }
 
@@ -340,10 +357,18 @@ static void textboxDraw(newtComponent c) {
     struct textbox * tb = c->data;
     int size;
 
+    if (!c->isMapped)
+	    return;
+
     if (tb->sb) {
 	size = tb->numLines - c->height;
-	newtScrollbarSet(tb->sb, tb->topLine, size ? size : 0);
-	tb->sb->ops->draw(tb->sb);
+	if (tb->isActive) {
+		newtScrollbarSet(tb->sb_act, tb->topLine, size ? size : 0);
+		tb->sb_act->ops->draw(tb->sb_act);
+	} else {
+		newtScrollbarSet(tb->sb, tb->topLine, size ? size : 0);
+		tb->sb->ops->draw(tb->sb);
+	}
     }
 
     SLsmg_set_color(NEWT_COLORSET_TEXTBOX);
@@ -354,6 +379,8 @@ static void textboxDraw(newtComponent c) {
 	newtGotorc(c->top + i, c->left);
 	SLsmg_write_string(tb->lines[i + tb->topLine]);
     }
+    /* put cursor at beginning of text for better accessibility */
+    newtGotorc(c->top, c->left);
 }
 
 static struct eventResult textboxEvent(newtComponent co, 
@@ -363,7 +390,11 @@ static struct eventResult textboxEvent(newtComponent co,
 
     er.result = ER_IGNORED;
 
-    if (ev.when == EV_EARLY && ev.event == EV_KEYPRESS && tb->sb) {
+    if (!tb->sb || ev.when == EV_EARLY || ev.when == EV_LATE)
+	return er;
+
+    switch(ev.event) {
+      case EV_KEYPRESS:
 	newtTrashScreen();
 	switch (ev.u.key) {
 	  case NEWT_KEY_UP:
@@ -395,8 +426,8 @@ static struct eventResult textboxEvent(newtComponent co,
 	    er.result = ER_SWALLOWED;
 	    break;
 	}
-    }
-    if (ev.when == EV_EARLY && ev.event == EV_MOUSE && tb->sb) {
+	break;
+      case EV_MOUSE:
 	/* Top scroll arrow */
 	if (ev.u.mouse.x == co->width && ev.u.mouse.y == co->top) {
 	    if (tb->topLine) tb->topLine--;
@@ -412,6 +443,17 @@ static struct eventResult textboxEvent(newtComponent co,
 	    
 	    er.result = ER_SWALLOWED;
 	}
+	break;
+      case EV_FOCUS:
+	tb->isActive = 1;
+	textboxDraw(co);
+	er.result = ER_SWALLOWED;
+	break;
+      case EV_UNFOCUS:
+	tb->isActive = 0;
+	textboxDraw(co);
+	er.result = ER_SWALLOWED;
+	break;
     }
     return er;
 }
@@ -420,6 +462,10 @@ static void textboxDestroy(newtComponent co) {
     int i;
     struct textbox * tb = co->data;
 
+    if (tb->sb)
+	tb->sb->ops->destroy(tb->sb);
+    if (tb->sb_act)
+	tb->sb_act->ops->destroy(tb->sb_act);
     for (i = 0; i < tb->numLines; i++) 
 	free(tb->lines[i]);
     free(tb->lines);
