@@ -44,20 +44,25 @@ static void usage(int err) {
 	       "\t--textbox <file> <height> <width>\n"
 	       "\t--menu <text> <height> <width> <listheight> [tag item] ...\n"
 	       "\t--checklist <text> <height> <width> <listheight> [tag item status]...\n"
-	       "\t--radiolist <text> <height> <width> <listheight> [tag item stautus]...\n"
+	       "\t--radiolist <text> <height> <width> <listheight> [tag item status]...\n"
 	       "\t--gauge <text> <height> <width> <percent>\n"
 	       "Options: (depend on box-option)\n"
 	       "\t--clear				clear screen on exit\n"
-	       "\t-defaultno			default no button\n"	
+	       "\t--defaultno			default no button\n"	
 	       "\t--default-item <string>		set default string\n"
 	       "\t--fb				use full buttons\n"
 	       "\t--nocancel			no cancel button\n"
+	       "\t--yes-button <text>		set text of yes button\n"
+	       "\t--no-button <text>		set text of no button\n"
+	       "\t--ok-button <text>		set text of ok button\n"
+	       "\t--cancel-button <text>		set text of cancel button\n"
 	       "\t--noitem			display tags only\n"
-	       "\t--separate-output <fd>		output one line at a time\n"
+	       "\t--separate-output		output one line at a time\n"
 	       "\t--output-fd <fd>		output to fd, not stdout\n"
 	       "\t--title <title>			display title\n"
 	       "\t--backtitle <backtitle>		display backtitle\n"
-	       "\t--scrolltext			force verical scrollbars\n\n"));
+	       "\t--scrolltext			force verical scrollbars\n"
+	       "\t--topleft			put window in top-left corner\n\n"));
     exit(err ? DLG_ERROR : 0 );
 }
 
@@ -65,10 +70,6 @@ static void print_version(void) {
     fprintf (stdout, _("whiptail (newt): %s\n"), VERSION);
 }
 	     
-static void handleSighup(int signum) {
-       exit(DLG_ERROR);
-}
-
 #if 0
 /* FIXME Copied from newt.c
  * Place somewhere better -- dialogboxes? -- amck
@@ -196,8 +197,8 @@ static void spaceForButtons(int * height, int * width, int count, int full) {
 
 static int menuSize(int * height, int * width, enum mode mode,
 		    poptContext options) {
-    char **    argv = poptGetArgs(options);
-    char * *    items = argv;
+    const char ** argv = poptGetArgs(options);
+    const char ** items = argv;
     int         h = 0;
     int         tagWidth = 0;
     int         descriptionWidth = 0;
@@ -323,7 +324,6 @@ int main(int argc, const char ** argv) {
     char * text;
     const char * nextArg;
     char * end;
-    struct sigaction sa;
     int height;
     int width;
     int fd = -1;
@@ -339,12 +339,17 @@ int main(int argc, const char ** argv) {
     int separateOutput = 0;
     int fullButtons = 0;
     int outputfd = 2;
+    int topLeft = 0;
     FILE *output = stderr;
     const char * result;
     const char ** selections, ** next;
     char * title = NULL;
     char *default_item = NULL;
     char * backtitle = NULL;
+    char * yes_button = NULL;
+    char * no_button = NULL;
+    char * ok_button = NULL;
+    char * cancel_button = NULL;
     int help = 0, version = 0;
     struct poptOption optionsTable[] = {
 	    { "backtitle", '\0', POPT_ARG_STRING, &backtitle, 0 },
@@ -367,17 +372,24 @@ int main(int argc, const char ** argv) {
 	    { "separate-output", '\0', 0, &separateOutput, 0 },
 	    { "title", '\0', POPT_ARG_STRING, &title, 0 },
 	    { "textbox", '\0', 0, 0, OPT_TEXTBOX },
+	    { "topleft", '\0', 0, &topLeft, 0 },
 	    { "yesno", '\0', 0, 0, OPT_YESNO },
 	    { "passwordbox", '\0', 0, 0, OPT_PASSWORDBOX },
 	    { "output-fd", '\0',  POPT_ARG_INT, &outputfd, 0 },
+	    { "yes-button", '\0', POPT_ARG_STRING, &yes_button, 0},
+	    { "no-button", '\0', POPT_ARG_STRING, &no_button, 0},
+	    { "ok-button", '\0', POPT_ARG_STRING, &ok_button, 0},
+	    { "cancel-button", '\0', POPT_ARG_STRING, &cancel_button, 0},
 	    { "help", 'h', 0,  &help, 0, NULL, NULL },
 	    { "version", 'v', 0, &version, 0, NULL, NULL },
 	    { 0, 0, 0, 0, 0 } 
     };
    
+#ifdef ENABLE_NLS
     setlocale (LC_ALL, "");
     bindtextdomain (PACKAGE, LOCALEDIR);
     textdomain (PACKAGE);
+#endif
 
     optCon = poptGetContext("whiptail", argc, argv, optionsTable, 0);
 
@@ -466,7 +478,8 @@ int main(int argc, const char ** argv) {
 
     if (mode == MODE_NONE) usage(WAS_ERROR);
 
-    if (!(text = poptGetArg(optCon))) usage(WAS_ERROR);
+    if (!(nextArg = poptGetArg(optCon))) usage(WAS_ERROR);
+    text = strdup(nextArg);
 
     if  (mode == MODE_TEXTBOX ) text = readTextFile(text);
 
@@ -484,10 +497,6 @@ int main(int argc, const char ** argv) {
 	if (open("/dev/tty", O_RDWR) != 0) perror("open /dev/tty");
     }
 
-    memset(&sa, 0, sizeof(sa));
-    sa.sa_handler = handleSighup;
-    sigaction(SIGWINCH, &sa, NULL);
-
     newtInit();
     newtCls();
 
@@ -500,11 +509,20 @@ int main(int argc, const char ** argv) {
     width -= 2;
     height -= 2;
 
-    newtOpenWindow((SLtt_Screen_Cols - width) / 2, 
-                   (SLtt_Screen_Rows - height) / 2, width, height, title);
+    newtOpenWindow(topLeft ? 1 : (SLtt_Screen_Cols - width) / 2,
+		   topLeft ? 1 : (SLtt_Screen_Rows - height) / 2, width, height, title);
     if (backtitle)
 	newtDrawRootText(0, 0, backtitle);
 
+    if (ok_button)
+	setButtonText(ok_button, BUTTON_OK);
+    if (cancel_button)
+	setButtonText(cancel_button, BUTTON_CANCEL);
+    if (yes_button)
+	setButtonText(yes_button, BUTTON_YES);
+    if (no_button)
+	setButtonText(no_button, BUTTON_NO);
+
     if (noCancel) flags |= FLAG_NOCANCEL;
     if (noItem) flags |= FLAG_NOITEM;
     if (noTags) flags |= FLAG_NOTAGS;
@@ -544,7 +562,8 @@ int main(int argc, const char ** argv) {
       case MODE_RADIOLIST:
 	rc = checkList(text, height, width, optCon, 1, flags, &selections);
 	if (rc == DLG_OKAY) {
-	    fprintf(output, "%s", selections[0]);
+	    if (selections[0])
+		fprintf(output, "%s", selections[0]);
 	    free(selections);
 	}
 	break;
